import importlib
import json
from functools import lru_cache
from typing import Dict

import stringcase
from linkml.utils.generator import Generator
from linkml.generators.jsonschemagen import JsonSchemaGenerator
from linkml.generators.pythongen import PythonGenerator

from linkml_validator.plugins.base import BasePlugin


@lru_cache()
def get_python_module(schema: str, generator: Generator = PythonGenerator, **kwargs) -> object:
    """
    Get Python representation of the schema.

    Args:
        schema: Path or URL to schema YAML
        generator: The generator to use to generate the Python module

    Returns:
        object: The Python module compiled from schema YAML

    """
    kwargs["schema"] = schema
    python_module = generator(**kwargs).compile_module()
    return python_module


@lru_cache()
def get_jsonschema(schema: str, py_target_class: object = None, generator: Generator = JsonSchemaGenerator, **kwargs) -> Dict:
    """
    Get JSONSchema representation of the schema.

    Args:
        schema: Path or URL to schema YAML
        py_target_class: The Python representation of the target class
        generator: The generator to use to generate the JSONSchema

    Returns:
        dict: The JSONSchema compiled from the schema YAML

    """
    if "mergeimports" not in kwargs:
        kwargs["mergeimports"] = True
    kwargs["schema"] = schema
    kwargs["top_class"] = py_target_class.class_name if py_target_class else None
    if "not_closed" not in kwargs:
        kwargs["not_closed"] = False
    jsonschemastr = generator(**kwargs).serialize()
    jsonschema_obj = json.loads(jsonschemastr)
    return jsonschema_obj


def import_plugin(plugin_module_name: str, plugin_class_name: str) -> BasePlugin:
    """
    Import a plugin class.

    Args:
        plugin_module_name: The name of the plugin module
        plugin_class_name: The name of the class in the plugin module

    Returns:
        BasePlugin: The plugin class

    """
    plugin_module = importlib.import_module(plugin_module_name)
    plugin_class = getattr(plugin_module, plugin_class_name)
    if not issubclass(plugin_class, BasePlugin):
        raise Exception(f"{plugin_module_name}.{plugin_class_name} must be a subclass of {BasePlugin}")
    return plugin_class


def camelcase_to_sentencecase(name: str) -> str:
    """
    Convert a given string from CamelCase to sentence case.

    Args:
        name: A string in CamelCase

    Returns:
        str: The converted string in sentence case

    """
    return stringcase.sentencecase(stringcase.snakecase(name)).lower()


def snakecase_to_sentencecase(name: str) -> str:
    """
    Convert a given string from snake_case to sentence case.

    Args:
        name: A string in snake_case

    Returns:
        str: The converted string in sentence case

    """
    return stringcase.sentencecase(name).lower()
